
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_BLEND
#define GR_SHQN_BLEND


#include "base.h"
#include "node.h"


	/** Pixel arithmetic specification node.
	    This node enables and specifies the pixel arithmetic function in
		the frame buffer.
		@see opengl
	 */
class grBlend : public grNode
{
public:
		/// Default constructor
	inline grBlend();
		/// Generates opengl calls for drawing
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// See grNode
	const char * Name() const;
		/// See grNode
	void Render( grStatus & status );
	
/// Function source factor, values: GL_ZERO, GL_ONE, GL_DST_COLOR, GL_ONE_MINUS_DST_COLOR, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA, and GL_SRC_ALPHA_SATURATE
	GLenum sfactor;
/// Function destination factor, values: GL_ZERO, GL_ONE, GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_DST_ALPHA, and GL_ONE_MINUS_DST_ALPHA
	GLenum dfactor;
		/// Enable or disable the pixel arithmetic
	GLboolean enabled;
 
private:
	static const char * name;
};


inline grBlend::grBlend()
{
	sfactor = GL_ZERO;
	dfactor = GL_ZERO;
	enabled = 0;
}


inline void grBlend::GL() const
{
	if(enabled)
	{
		glEnable(GL_BLEND);
		glBlendFunc(sfactor,dfactor);
	}
	else
	{
		glDisable(GL_BLEND);
	}
}


inline void grBlend::Get()
{
	glGet(GL_BLEND_SRC, (int *)&sfactor);
	glGet(GL_BLEND_DST, (int *)&dfactor);
	enabled = glIsEnabled( GL_BLEND );
}



#endif