

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_CALLBACK
#define GR_SHQN_CALLBACK


#include "node.h"

	/** @name Callback system
	 */
	//@{

	/** Callback function type definition
	    @see grCallback, grStatus
	 */
typedef void grCallbackFun( const grStatus & status );

	/** Provides custom behavior during actions.
		This node provides a general mechanism for inserting callback
		functions into a scene graph. The callback function is specified
		with the method SetCallback. The user may access the current
		traversal status of the current action.
		@see grTree, grRender, grCallbackFun, grStatus
	 */
class grCallback : public grNode
{
public:
		/// Default constructor 
	inline grCallback();
		/// Generates opengl calls for drawing
	inline void GL() const;
		/// se grNode
	const char * Name() const;
		/// se grNode
	void Render( grStatus & status );
		/// se grNode
	void Select( grStatus & status );

		/// Set the pointer to callback function
	inline void SetCallback( grCallbackFun * const cb );

    grCallbackFun * callback;

private:
	static const char * name;
};


inline grCallback::grCallback()
{
	callback = 0;
}


inline void grCallback::GL() const
{
	// nothing	
}


inline void grCallback::SetCallback( grCallbackFun * const cb )
{
	callback = cb;
}

	//@}

#endif