

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_CLEARVALUE
#define GR_SHQN_CLEARVALUE


#include "base.h"
#include "node.h"
#include "color.h"

	/** Clear values specification.
	    This node specifies clear values for the glClear function.
		@see opengl
	 */
class grClearValue : public grNode
{
public:
		/// Default constructor
	inline grClearValue();
		/// Generates opengl calls for drawing
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// see grNode
	const char * Name() const;
		/// see grNode
	void Render( grStatus & status );
	
		/// clear value for accumulation buffer
	grColor4f accum;
		/// clear value for frame buffer
	grColor4f color;
		/// clear value for depth buffer (z-buffer)
	GLclampd  depth;
		/// clear value for color-index buffer 
	GLfloat   index;
		/// clear value for stencil buffer
	GLint     stencil;

private:
	static const char * name;
};


inline grClearValue::grClearValue()
{
	accum.Set(0.0f,0.0f,0.0f,0.0f);
	color.Set(0.0f,0.0f,0.0f,0.0f);
	depth   = GLclampd(1.0);
	index   = 0;
	stencil = 0;
}


inline void grClearValue::GL() const
{
	glClearAccum(accum[0],accum[1],accum[2],accum[3]);
	glClearColor(color[0],color[1],color[2],color[3]);
	glClearDepth(depth);
	glClearIndex(index);
	glClearStencil(stencil);
}


inline void grClearValue::Get()
{
	glGet(GL_ACCUM_CLEAR_VALUE,accum.color);
	glGet(GL_COLOR_CLEAR_VALUE,color.color);
	glGet(GL_DEPTH_CLEAR_VALUE,&depth);
	glGet(GL_INDEX_CLEAR_VALUE,&index);
	glGet(GL_STENCIL_CLEAR_VALUE,&stencil);
}


#endif