
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD2D
#define GR_SHQN_COORD2D


#include <stdio.h>
#include <math.h>


	

	/** Bidimensional point.
		Bidimensional point (double), see Coord3f for the member definition, except
		cross product.
	  */
class Coord2d
{
public:
    
    double x,y;

    inline Coord2d ( void ) { }
    inline Coord2d ( const double nx, const double ny );
    inline Coord2d ( Coord2d const & v);
    inline Coord2d & operator = ( Coord2d const & v);
    inline Coord2d & Set( const double X, const double Y );
    inline Coord2d & Get( double & X, double & Y );
    inline double & operator [] ( const int i );
    inline Coord2d operator + ( Coord2d const & v) const;
    inline Coord2d operator - ( Coord2d const & v) const;
    inline Coord2d operator * ( const double s ) const;
    inline Coord2d operator / ( const double s ) const;
    inline double Norm( void ) const;
    inline double Norm2( void ) const;
    inline double operator * ( Coord2d const & v ) const;      // Dot product
    inline Coord2d & operator += ( Coord2d const & v);
    inline Coord2d & operator -= ( Coord2d const & v);
    inline Coord2d & operator *= ( const double s );
    inline Coord2d & operator /= ( const double s );
    inline Coord2d & Scale2( const double sx, const double sy );
    inline Coord2d & Normalize( void );
    inline bool operator == ( Coord2d const & v ) const ;
    inline bool operator != ( Coord2d const & v ) const ;
	inline bool  eq ( Coord2d const & v, const double eps ) const ;
	inline bool neq ( Coord2d const & v, const double eps ) const ;
    inline bool operator <  ( Coord2d const & v ) const ;
    inline bool operator >  ( Coord2d const & v ) const ;
    inline bool operator <= ( Coord2d const & v ) const ;
    inline bool operator >= ( Coord2d const & v ) const ;
    inline int qsortcomp( Coord2d const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline double Hash( void ) const;			// range [0-1)
    inline double Dist( Coord2d const & p ) const;
};


inline Coord2d	operator - ( Coord2d const & v );
inline Coord2d	operator * ( const double s, Coord2d const & v );
inline double	Dist( Coord2d const & v1, Coord2d const & v2 );
inline Coord2d & Normalize( Coord2d & v );
inline double	Norm( Coord2d const & v );
inline double   Norm2( Coord2d const & v );


inline Coord2d::Coord2d ( const double nx, const double ny )
{
    x = nx;
    y = ny;
}


inline Coord2d::Coord2d( Coord2d const & v)
{
    x= v.x;
    y= v.y;
}


inline Coord2d & Coord2d::operator =( Coord2d const & v)
{
    x= v.x;
    y= v.y;
    return *this;
}


inline Coord2d & Coord2d::Set( const double X, const double Y )
{
    x= X;
    y= Y;
    return *this;
} 


inline Coord2d & Coord2d::Get( double & X, double & Y )
{
    X= x;
    Y= y;
    return *this;
}


inline double & Coord2d::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
    }
    return x;
}


inline Coord2d Coord2d::operator + ( Coord2d const & v) const
{
    return Coord2d( x+v.x, y+v.y );
}


inline Coord2d Coord2d::operator - ( Coord2d const & v) const
{
    return Coord2d( x-v.x, y-v.y );
}


inline Coord2d Coord2d::operator * ( const double s ) const
{
    return Coord2d( x*s, y*s );
}


inline Coord2d Coord2d::operator / ( const double s ) const
{
    return Coord2d( x / s, y / s );
} 


inline double Coord2d::operator * ( Coord2d const & v ) const
{
    return ( x*v.x + y*v.y );
}


inline double Coord2d::Norm( void ) const
{
    return sqrt( x*x + y*y );
}


inline double Coord2d::Norm2( void ) const
{
    return ( x*x + y*y );
}


inline Coord2d & Coord2d::operator += ( Coord2d const & v)
{
    x += v.x;
    y += v.y;
    return *this;
}


inline Coord2d & Coord2d::operator -= ( Coord2d const & v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

inline Coord2d & Coord2d::operator *= ( const double s )
{
    x *= s;
    y *= s;
    return *this;
}


inline Coord2d & Coord2d::operator /= ( const double s )
{
    x /= s;
    y /= s;
    return *this;
}   


inline Coord2d & Coord2d::Scale2( const double sx, const double sy )
{
    x /= sx;
    y /= sy;
    return *this;
}


inline bool Coord2d::operator == ( Coord2d const & v ) const
{
    return x==v.x && y==v.y;
}   


inline bool Coord2d::eq( Coord2d const & v, const double eps ) const
{
    return ( fabs(x-v.x)<eps &&
			 fabs(y-v.y)<eps );
}   


inline bool Coord2d::operator != ( Coord2d const & v ) const
{
    return x!=v.x || y!=v.y;
}


inline bool Coord2d::neq( Coord2d const & v, const double eps ) const {
    return ( fabs(x-v.x)>eps ||
	         fabs(y-v.y)>eps );
}


inline bool Coord2d::operator <  ( Coord2d const & v ) const
{
    return (x!=v.x)?(x<v.x):(y<v.y);
}


inline bool Coord2d::operator >  ( Coord2d const & v ) const
{
    return (x!=v.x)?(x>v.x):(y>v.y);
}


inline bool Coord2d::operator <= ( Coord2d const & v ) const
{
    return (x!=v.x)?(x< v.x):(y< v.y);
}


inline bool Coord2d::operator >= ( Coord2d const & v ) const
{
    return (x!=v.x)?(x> v.x):(y> v.y);
}


inline int Coord2d::qsortcomp( Coord2d const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       0;
}


inline Coord2d & Coord2d::Normalize( void )
{
    double n = sqrt(x*x + y*y);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
    }
    return *this;
}


inline int Coord2d::Read ( FILE *fp )
{
    float nx,ny;
    int r = fscanf(fp,"%f%f",&nx,&ny);
    x = double(nx);
    y = double(ny);
    return r;
}


inline int Coord2d::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%g %g ",x,y));
}    


inline int Coord2d::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(double),1,fp);
    r &= fread(&y,sizeof(double),1,fp);
    return r;
}


inline int Coord2d::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(double),1,fp);
    r &= fwrite(&y,sizeof(double),1,fp);
    return r;
}


inline double Coord2d::Hash( void ) const
{
    double t;
    return modf( log(x*x+3*y*y),&t );
}


inline double operator % ( Coord2d const & v1, Coord2d const & v2 )
{
    return (double) acos( (v1*v2)/(v1.Norm()*v2.Norm()) );
}


inline Coord2d operator - ( Coord2d const & v ){
    return Coord2d( -v.x, -v.y );
}


inline Coord2d operator * ( const double s, Coord2d const & v )
{
    return Coord2d( v.x * s, v.y * s );
}


inline double Norm( Coord2d const & v )
{
    return sqrt( v.x*v.x + v.y*v.y );
}


inline double Norm2( Coord2d const & v )
{
    return ( v.x*v.x + v.y*v.y );
}


inline Coord2d & Normalize( Coord2d & v )
{
    double n = sqrt( v.x*v.x + v.y*v.y );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
    }
    return v;
}


inline double Dist( Coord2d const & v1,Coord2d const & v2 )
{
    return Norm(v1-v2);
}


inline double Coord2d::Dist( Coord2d const & p ) const
{
    return (*this-p).Norm();
}



#endif
