
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_COORD3D
#define GR_SHQN_COORD3D


#include <stdio.h>
#include <math.h>


//namespace gcr
//{
	
	/** Tridimensional point.
		Tridimensional point (double), see Coord3f for the member definition.
	  */
class Coord3d
{
public:
    
    double x,y,z;

    inline Coord3d ( void ) { }
    inline Coord3d ( const double nx, const double ny, const double nz );
    inline Coord3d ( Coord3d const & v);
    inline Coord3d & operator = ( Coord3d const & v);
    inline Coord3d & Set( const double X, const double Y, const double Z );
    inline Coord3d & Get( double & X, double & Y, double & Z );
    inline double & operator [] ( const int i );
    inline Coord3d operator + ( Coord3d const & v) const;
    inline Coord3d operator - ( Coord3d const & v) const;
    inline Coord3d operator * ( const double s ) const;
    inline Coord3d operator / ( const double s ) const;
    inline double Norm( void ) const;
    inline double Norm2( void ) const;
    inline double operator * ( Coord3d const & v ) const;      // Dot product
    inline Coord3d operator ^ ( Coord3d const & v ) const;   // Cross Product
    inline Coord3d & operator += ( Coord3d const & v);
    inline Coord3d & operator -= ( Coord3d const & v);
    inline Coord3d & operator *= ( const double s );
    inline Coord3d & operator /= ( const double s );
    inline Coord3d & Scale3( const double sx, const double sy, const double sz );
    inline Coord3d & Normalize( void );
    inline bool operator == ( Coord3d const & v ) const ;
    inline bool operator != ( Coord3d const & v ) const ;
	inline bool  eq ( Coord3d const & v, const double eps ) const ;
	inline bool neq ( Coord3d const & v, const double eps ) const ;
    inline bool operator <  ( Coord3d const & v ) const ;
    inline bool operator >  ( Coord3d const & v ) const ;
    inline bool operator <= ( Coord3d const & v ) const ;
    inline bool operator >= ( Coord3d const & v ) const ;
    inline int qsortcomp( Coord3d const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline double Hash( void ) const;			// range [0-1)
    inline double Dist( Coord3d const & p ) const;
};


inline double	operator % ( Coord3d const & v1, Coord3d const & v2 );
inline Coord3d	operator - ( Coord3d const & v );
inline Coord3d	operator * ( const double s, Coord3d const & v );
inline double	Dist( Coord3d const & v1, Coord3d const & v2 );
inline Coord3d & Normalize( Coord3d & v );
inline double	Norm( Coord3d const & v );
inline double   Norm2( Coord3d const & v );


inline Coord3d::Coord3d ( const double nx, const double ny, const double nz )
{
    x = nx;
    y = ny;
    z = nz;
}


inline Coord3d::Coord3d( Coord3d const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
}


inline Coord3d & Coord3d::operator =( Coord3d const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
    return *this;
}


inline Coord3d & Coord3d::Set( const double X, const double Y, const double Z )
{
    x= X;
    y= Y;
    z= Z;
    return *this;
} 


inline Coord3d & Coord3d::Get( double & X, double & Y, double & Z )
{
    X= x;
    Y= y;
    Z= z;
    return *this;
}


inline double & Coord3d::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
    }
    return x;
}


inline Coord3d Coord3d::operator + ( Coord3d const & v) const
{
    return Coord3d( x+v.x, y+v.y, z+v.z );
}


inline Coord3d Coord3d::operator - ( Coord3d const & v) const
{
    return Coord3d( x-v.x, y-v.y, z-v.z );
}


inline Coord3d Coord3d::operator * ( const double s ) const
{
    return Coord3d( x*s, y*s, z*s );
}


inline Coord3d Coord3d::operator / ( const double s ) const
{
    return Coord3d( x / s, y / s, z / s );
} 


inline double Coord3d::operator * ( Coord3d const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z );
}


inline Coord3d Coord3d::operator ^ ( Coord3d const & v ) const
{
    return Coord3d(
        y*v.z - z*v.y,
        z*v.x - x*v.z,
        x*v.y - y*v.x
    );
}


inline double Coord3d::Norm( void ) const
{
    return sqrt( x*x + y*y + z*z );
}


inline double Coord3d::Norm2( void ) const
{
    return ( x*x + y*y + z*z );
}


inline Coord3d & Coord3d::operator += ( Coord3d const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
}


inline Coord3d & Coord3d::operator -= ( Coord3d const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
    return *this;
}

inline Coord3d & Coord3d::operator *= ( const double s )
{
    x *= s;
    y *= s;
    z *= s;
    return *this;
}


inline Coord3d & Coord3d::operator /= ( const double s )
{
    x /= s;
    y /= s;
    z /= s;
    return *this;
}   


inline Coord3d & Coord3d::Scale3( const double sx, const double sy, const double sz )
{
    x /= sx;
    y /= sy;
    z /= sz;
    return *this;
}


inline bool Coord3d::operator == ( Coord3d const & v ) const
{
    return x==v.x && y==v.y && z==v.z;
}   


inline bool Coord3d::eq( Coord3d const & v, const double eps ) const
{
    return ( fabs(x-v.x)<eps &&
			 fabs(y-v.y)<eps &&
	         fabs(z-v.z)<eps    );
}   


inline bool Coord3d::operator != ( Coord3d const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z;
}


inline bool Coord3d::neq( Coord3d const & v, const double eps ) const {
    return ( fabs(x-v.x)>eps ||
	         fabs(y-v.y)>eps ||
	         fabs(z-v.z)>eps   );
}


inline bool Coord3d::operator <  ( Coord3d const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z<v.z);
}


inline bool Coord3d::operator >  ( Coord3d const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z>v.z);
}


inline bool Coord3d::operator <= ( Coord3d const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z<=v.z);
}


inline bool Coord3d::operator >= ( Coord3d const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z>=v.z);
}


inline int Coord3d::qsortcomp( Coord3d const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
	       0;
}


inline Coord3d & Coord3d::Normalize( void )
{
    double n = sqrt(x*x + y*y + z*z);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
		z /= n;
    }
    return *this;
}


inline int Coord3d::Read ( FILE *fp )
{
    float nx,ny,nz;
    int r = fscanf(fp,"%f%f%f",&nx,&ny,&nz);
    x = double(nx);
    y = double(ny);
    z = double(nz);
    return r;
}


inline int Coord3d::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%g %g %g ",x,y,z));
}    


inline int Coord3d::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(double),1,fp);
    r &= fread(&y,sizeof(double),1,fp);
    r &= fread(&z,sizeof(double),1,fp);
    return r;
}


inline int Coord3d::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(double),1,fp);
    r &= fwrite(&y,sizeof(double),1,fp);
    r &= fwrite(&z,sizeof(double),1,fp);
    return r;
}


inline double Coord3d::Hash( void ) const
{
    double t;
    return modf( log(x*x+3*y*y+5*z*z),&t );
}


inline double operator % ( Coord3d const & v1, Coord3d const & v2 )
{
    return (double) acos( (v1*v2)/(v1.Norm()*v2.Norm()) );
}


inline Coord3d operator - ( Coord3d const & v ){
    return Coord3d( -v.x, -v.y, -v.z );
}


inline Coord3d operator * ( const double s, Coord3d const & v )
{
    return Coord3d( v.x * s, v.y * s, v.z * s );
}


inline double Norm( Coord3d const & v )
{
    return sqrt( v.x*v.x + v.y*v.y + v.z*v.z );
}


inline double Norm2( Coord3d const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z );
}


inline Coord3d & Normalize( Coord3d & v )
{
    double n = sqrt( v.x*v.x + v.y*v.y + v.z*v.z );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
        v.z /= n;
    }
    return v;
}


inline double Dist( Coord3d const & v1,Coord3d const & v2 )
{
    return Norm(v1-v2);
}


inline double Coord3d::Dist( Coord3d const & p ) const
{
    return (*this-p).Norm();
}


//}	// End namespace


#endif
