
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD3F
#define GR_SHQN_COORD3F


#include <stdio.h>
#include <math.h>


//namespace gcr
//{
	
	/** Tridimensional point.
		Tridimensional point (float),
	  */
class Coord3f
{
public:
		/// 3D components
    float x,y,z;

		/// Default constructor
    inline Coord3f ( void ) { }
		/// Costructor from components
    inline Coord3f ( const float nx, const float ny, const float nz );
		/// Copy contructor
    inline Coord3f ( Coord3f const & v);
		/// Assignement operator
    inline Coord3f & operator = ( Coord3f const & v);
		/// Set the point components
    inline Coord3f & Set( const float X, const float Y, const float Z );
		/// Get the point components
    inline Coord3f & Get( float & X, float & Y, float & Z );
		/// Component selection operator (i.e. v[1] = v.y)
    inline float & operator [] ( const int i );
		/// Constatnt component selection operator
	inline const float & operator [] ( const int i ) const;
		/// Sum operator
    inline Coord3f operator + ( Coord3f const & v) const;
		/// Subtraction operator
    inline Coord3f operator - ( Coord3f const & v) const;
		/// Per scalar product operator
    inline Coord3f operator * ( const float s ) const;
		/// Per scalar division operator
    inline Coord3f operator / ( const float s ) const;
		/// Normal of vector
    inline float Norm( void ) const;
		/// Squared normal 
    inline float Norm2( void ) const;
		/// Dot product
    inline float operator * ( Coord3f const & v ) const;      // Dot product
		/// Cross product
    inline Coord3f operator ^ ( Coord3f const & v ) const;   // Cross Product
		/// Vector increment
    inline Coord3f & operator += ( Coord3f const & v);
		/// Vector decrement
    inline Coord3f & operator -= ( Coord3f const & v);
		/// Per scalar product
    inline Coord3f & operator *= ( const float s );
		/// Per scalar division
    inline Coord3f & operator /= ( const float s );
		/// Component scale
    inline Coord3f & Scale3( const float sx, const float sy, const float sz );
		/// Vector Normalisation
    inline Coord3f & Normalize( void );
		/// Equal operator
    inline bool operator == ( Coord3f const & v ) const ;
		/// Not equal operator
    inline bool operator != ( Coord3f const & v ) const ;
		/// Equal operator with tolerance eps
	inline bool  eq ( Coord3f const & v, const float eps ) const ;
		/// Not equal operator with tolerance eps
	inline bool neq ( Coord3f const & v, const float eps ) const ;
		/// Ordering operator
    inline bool operator <  ( Coord3f const & v ) const ;
		/// Ordering operator
    inline bool operator >  ( Coord3f const & v ) const ;
		/// Ordering operator
    inline bool operator <= ( Coord3f const & v ) const ;
		/// Ordering operator
    inline bool operator >= ( Coord3f const & v ) const ;
    inline int qsortcomp( Coord3f const & v ) const;
		/// Read an object (ascii mode)
    inline int Read ( FILE *fp ) ;
		/// Save an object, (ascii mode)
    inline int Write( FILE *fp ) const;
		/// Read an object (binary mode)
    inline int ReadBin( FILE *fp ) ;
		/// Save an object, (bynary mode)
    inline int WriteBin( FILE *fp ) const;
		/// Hanshing function: return a value in the (0,1) range
    inline double Hash( void ) const;
		/// Euclidean distance between points
    inline float Dist( Coord3f const & p ) const;
};

	/** @name Coord3f external functions.
	    Coord3f external functions.
	 */
	//@{

	/// Angle between vectors
inline float	operator % ( Coord3f const & v1, Coord3f const & v2 );
	/// unary minus
inline Coord3f	operator - ( Coord3f const & v );
	/// Per scalar product
inline Coord3f	operator * ( const float s, Coord3f const & v );
	/// Euclidean distance
inline float	Dist( Coord3f const & v1, Coord3f const & v2 );
	/// Vector normalisation
inline Coord3f & Normalize( Coord3f & v );
	/// Vector normal
inline float	Norm( Coord3f const & v );
	/// Vector squared normal
inline float    Norm2( Coord3f const & v );

	//@}

inline Coord3f::Coord3f ( const float nx, const float ny, const float nz )
{
    x = nx;
    y = ny;
    z = nz;
}


inline Coord3f::Coord3f( Coord3f const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
}


inline Coord3f & Coord3f::operator =( Coord3f const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
    return *this;
}


inline Coord3f & Coord3f::Set( const float X, const float Y, const float Z )
{
    x= X;
    y= Y;
    z= Z;
    return *this;
} 


inline Coord3f & Coord3f::Get( float & X, float & Y, float & Z )
{
    X= x;
    Y= y;
    Z= z;
    return *this;
}


inline float & Coord3f::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
    }
    return x;
}


inline const float & Coord3f::operator [] ( const int i ) const
{
	switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
    }
    return x;
}


inline Coord3f Coord3f::operator + ( Coord3f const & v) const
{
    return Coord3f( x+v.x, y+v.y, z+v.z );
}


inline Coord3f Coord3f::operator - ( Coord3f const & v) const
{
    return Coord3f( x-v.x, y-v.y, z-v.z );
}


inline Coord3f Coord3f::operator * ( const float s ) const
{
    return Coord3f( x*s, y*s, z*s );
}


inline Coord3f Coord3f::operator / ( const float s ) const
{
    return Coord3f( x / s, y / s, z / s );
} 


inline float Coord3f::operator * ( Coord3f const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z );
}


inline Coord3f Coord3f::operator ^ ( Coord3f const & v ) const
{
    return Coord3f(
        y*v.z - z*v.y,
        z*v.x - x*v.z,
        x*v.y - y*v.x
    );
}


inline float Coord3f::Norm( void ) const
{
    return sqrtf( x*x + y*y + z*z );
}


inline float Coord3f::Norm2( void ) const
{
    return ( x*x + y*y + z*z );
}


inline Coord3f & Coord3f::operator += ( Coord3f const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
}


inline Coord3f & Coord3f::operator -= ( Coord3f const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
    return *this;
}

inline Coord3f & Coord3f::operator *= ( const float s )
{
    x *= s;
    y *= s;
    z *= s;
    return *this;
}


inline Coord3f & Coord3f::operator /= ( const float s )
{
    x /= s;
    y /= s;
    z /= s;
    return *this;
}   


inline Coord3f & Coord3f::Scale3( const float sx, const float sy, const float sz )
{
    x /= sx;
    y /= sy;
    z /= sz;
    return *this;
}


inline bool Coord3f::operator == ( Coord3f const & v ) const
{
    return x==v.x && y==v.y && z==v.z;
}   


inline bool Coord3f::eq( Coord3f const & v, const float eps ) const
{
    return ( fabsf(x-v.x)<eps &&
			 fabsf(y-v.y)<eps &&
	         fabsf(z-v.z)<eps    );
}   


inline bool Coord3f::operator != ( Coord3f const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z;
}


inline bool Coord3f::neq( Coord3f const & v, const float eps ) const {
    return ( fabsf(x-v.x)>eps ||
	         fabsf(y-v.y)>eps ||
	         fabsf(z-v.z)>eps   );
}


inline bool Coord3f::operator <  ( Coord3f const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z<v.z);
}


inline bool Coord3f::operator >  ( Coord3f const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z>v.z);
}


inline bool Coord3f::operator <= ( Coord3f const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z<=v.z);
}


inline bool Coord3f::operator >= ( Coord3f const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z>=v.z);
}


inline int Coord3f::qsortcomp( Coord3f const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
	       0;
}


inline Coord3f & Coord3f::Normalize( void )
{
    float n = sqrtf(x*x + y*y + z*z);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
		z /= n;
    }
    return *this;
}


inline int Coord3f::Read ( FILE *fp )
{
    return fscanf(fp,"%f%f%f",&x,&y,&z);
}


inline int Coord3f::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%g %g %g ",x,y,z));
}    


inline int Coord3f::ReadBin( FILE *fp ) {
    int r = 1;
    
    r &= fread(&x,sizeof(float),1,fp);
    r &= fread(&y,sizeof(float),1,fp);
    r &= fread(&z,sizeof(float),1,fp);
    return r;
}


inline int Coord3f::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(float),1,fp);
    r &= fwrite(&y,sizeof(float),1,fp);
    r &= fwrite(&z,sizeof(float),1,fp);
    return r;
}


inline double Coord3f::Hash( void ) const
{
    double t;
    return modf( log(x*x+3*y*y+5*z*z),&t );
}


inline float operator % ( Coord3f const & v1, Coord3f const & v2 )
{
    return (float) acos( (v1*v2)/(v1.Norm()*v2.Norm()) );
}


inline Coord3f operator - ( Coord3f const & v ){
    return Coord3f( -v.x, -v.y, -v.z );
}


inline Coord3f operator * ( const float s, Coord3f const & v )
{
    return Coord3f( v.x * s, v.y * s, v.z * s );
}


inline float Norm( Coord3f const & v )
{
    return sqrtf( v.x*v.x + v.y*v.y + v.z*v.z );
}


inline float Norm2( Coord3f const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z );
}


inline Coord3f & Normalize( Coord3f & v )
{
    float n = sqrtf( v.x*v.x + v.y*v.y + v.z*v.z );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
        v.z /= n;
    }
    return v;
}


inline float Dist( Coord3f const & v1,Coord3f const & v2 )
{
    return Norm(v1-v2);
}


inline float Coord3f::Dist( Coord3f const & p ) const
{
    return (*this-p).Norm();
}


//}	// End namespace


#endif
