
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD4F
#define GR_SHQN_COORD4F


#include <stdio.h>
#include <math.h>


//namespace gcr
//{
	
	/** Quadrimensional point.
		Quadrimensional point (float), see Coord3f for the member definition, except
		cross product.
	  */
class Coord4f
{
public:
    
    float x,y,z,w;

    inline Coord4f ( void ) { }
    inline Coord4f ( const float nx, const float ny,
		             const float nz, const float nw );
    inline Coord4f ( Coord4f const & v);
    inline Coord4f & operator = ( Coord4f const & v);
    inline Coord4f & Set( const float X, const float Y, const float Z, const float W );
    inline Coord4f & Get( float & X, float & Y, float & Z, float & W );
    inline float & operator [] ( const int i );
    inline Coord4f operator + ( Coord4f const & v) const;
    inline Coord4f operator - ( Coord4f const & v) const;
    inline Coord4f operator * ( const float s ) const;
    inline Coord4f operator / ( const float s ) const;
    inline float Norm( void ) const;
    inline float Norm2( void ) const;
    inline float operator * ( Coord4f const & v ) const;      // Dot product
    inline Coord4f & operator += ( Coord4f const & v);
    inline Coord4f & operator -= ( Coord4f const & v);
    inline Coord4f & operator *= ( const float s );
    inline Coord4f & operator /= ( const float s );
    inline Coord4f & Scale4( const float sx, const float sy,
		                     const float sz, const float sw );
    inline Coord4f & Normalize( void );
    inline bool operator == ( Coord4f const & v ) const ;
    inline bool operator != ( Coord4f const & v ) const ;
	inline bool  eq ( Coord4f const & v, const float eps ) const ;
	inline bool neq ( Coord4f const & v, const float eps ) const ;
    inline bool operator <  ( Coord4f const & v ) const ;
    inline bool operator >  ( Coord4f const & v ) const ;
    inline bool operator <= ( Coord4f const & v ) const ;
    inline bool operator >= ( Coord4f const & v ) const ;
    inline int qsortcomp( Coord4f const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline double Hash( void ) const;			// range [0-1)
    inline float Dist( Coord4f const & p ) const;
};


inline Coord4f	operator - ( Coord4f const & v );
inline Coord4f	operator * ( const float s, Coord4f const & v );
inline float	Dist( Coord4f const & v1, Coord4f const & v2 );
inline Coord4f & Normalize( Coord4f & v );
inline float	Norm( Coord4f const & v );
inline float    Norm2( Coord4f const & v );


inline Coord4f::Coord4f ( const float nx, const float ny,
						  const float nz, const float nw )
{
    x = nx;
    y = ny;
    z = nz;
	w = nw;
}


inline Coord4f::Coord4f( Coord4f const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
}


inline Coord4f & Coord4f::operator =( Coord4f const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
    return *this;
}


inline Coord4f & Coord4f::Set( const float X, const float Y,
							   const float Z, const float W )
{
    x= X;
    y= Y;
    z= Z;
	w= W;
    return *this;
} 


inline Coord4f & Coord4f::Get( float & X, float & Y, float & Z, float & W )
{
    X= x;
    Y= y;
    Z= z;
	W= w;
    return *this;
}


inline float & Coord4f::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
		case 3 : return w;
    }
    return x;
}


inline Coord4f Coord4f::operator + ( Coord4f const & v) const
{
    return Coord4f( x+v.x, y+v.y, z+v.z, w+v.w );
}


inline Coord4f Coord4f::operator - ( Coord4f const & v) const
{
    return Coord4f( x-v.x, y-v.y, z-v.z, w-v.w );
}


inline Coord4f Coord4f::operator * ( const float s ) const
{
    return Coord4f( x*s, y*s, z*s, w*s );
}


inline Coord4f Coord4f::operator / ( const float s ) const
{
    return Coord4f( x / s, y / s, z / s, w / s );
} 


inline float Coord4f::operator * ( Coord4f const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z +w*v.w );
}


inline float Coord4f::Norm( void ) const
{
    return sqrtf( x*x + y*y + z*z + w*w );
}


inline float Coord4f::Norm2( void ) const
{
    return ( x*x + y*y + z*z *w*w );
}


inline Coord4f & Coord4f::operator += ( Coord4f const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
	w += v.w;
    return *this;
}


inline Coord4f & Coord4f::operator -= ( Coord4f const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
	w -= v.w;
    return *this;
}

inline Coord4f & Coord4f::operator *= ( const float s )
{
    x *= s;
    y *= s;
    z *= s;
	w *= s;
    return *this;
}


inline Coord4f & Coord4f::operator /= ( const float s )
{
    x /= s;
    y /= s;
    z /= s;
	w /= s;
    return *this;
}   


inline Coord4f & Coord4f::Scale4( const float sx, const float sy,
								  const float sz, const float sw )
{
    x /= sx;
    y /= sy;
    z /= sz;
	w /= sw;
    return *this;
}


inline bool Coord4f::operator == ( Coord4f const & v ) const
{
    return x==v.x && y==v.y && z==v.z && w==v.w;
}   


inline bool Coord4f::eq( Coord4f const & v, const float eps ) const
{
    return ( fabsf(x-v.x)<eps &&
			 fabsf(y-v.y)<eps &&
	         fabsf(z-v.z)<eps &&
			 fabsf(w-v.w)<eps );
}   


inline bool Coord4f::operator != ( Coord4f const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z || w!=v.w;
}


inline bool Coord4f::neq( Coord4f const & v, const float eps ) const {
    return ( fabsf(x-v.x)>eps ||
	         fabsf(y-v.y)>eps ||
	         fabsf(z-v.z)>eps ||
			 fabsf(w-v.w)>eps  );
}


inline bool Coord4f::operator <  ( Coord4f const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z!=v.z)?(z<v.z):
		            (w<v.w);
}


inline bool Coord4f::operator >  ( Coord4f const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z!=v.z)?(z>v.z):
		            (w>v.w);
}


inline bool Coord4f::operator <= ( Coord4f const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z!=v.z)?(z< v.z):
					(w<=v.w);
}


inline bool Coord4f::operator >= ( Coord4f const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z!=v.z)?(z> v.z):
					(w>=v.w);
}


inline int Coord4f::qsortcomp( Coord4f const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
		   (w!=v.w)?( (w>v.w)?1:-1 ):
	       0;
}


inline Coord4f & Coord4f::Normalize( void )
{
    float n = sqrtf(x*x + y*y + z*z + w*w);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
		z /= n;
		w /= n;
    }
    return *this;
}


inline int Coord4f::Read ( FILE *fp )
{
    return fscanf(fp,"%f%f%f%f",&x,&y,&z,&w);
}


inline int Coord4f::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%g %g %g %g ",x,y,z,w));
}    


inline int Coord4f::ReadBin( FILE *fp ) {
    int r = 1;
    
    r &= fread(&x,sizeof(float),1,fp);
    r &= fread(&y,sizeof(float),1,fp);
    r &= fread(&z,sizeof(float),1,fp);
	r &= fread(&w,sizeof(float),1,fp);
    return r;
}


inline int Coord4f::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(float),1,fp);
    r &= fwrite(&y,sizeof(float),1,fp);
    r &= fwrite(&z,sizeof(float),1,fp);
	r &= fwrite(&w,sizeof(float),1,fp);
    return r;
}


inline double Coord4f::Hash( void ) const
{
    double t;
    return modf( log(x*x+3*y*y+5*z*z+7*w*w),&t );
}


inline Coord4f operator - ( Coord4f const & v ){
    return Coord4f( -v.x, -v.y, -v.z, -v.w );
}


inline Coord4f operator * ( const float s, Coord4f const & v )
{
    return Coord4f( v.x * s, v.y * s, v.z * s, v.w * s );
}


inline float Norm( Coord4f const & v )
{
    return sqrtf( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w);
}


inline float Norm2( Coord4f const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
}


inline Coord4f & Normalize( Coord4f & v )
{
    float n = sqrtf( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
        v.z /= n;
    }
    return v;
}


inline float Dist( Coord4f const & v1,Coord4f const & v2 )
{
    return Norm(v1-v2);
}


inline float Coord4f::Dist( Coord4f const & p ) const
{
    return (*this-p).Norm();
}


//}	// End namespace


#endif
