
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD4I
#define GR_SHQN_COORD4I


#include <stdio.h>


//namespace gcr
//{
	
	/** Quadrimensional point.
		Quadrimensional point (int), see Coord3f for the member definition, except
		cross product.
	  */
class Coord4i
{
public:
    
    int x,y,z,w;

    inline Coord4i ( void ) { }
    inline Coord4i ( const int nx, const int ny,
		             const int nz, const int nw );
    inline Coord4i ( Coord4i const & v);
    inline Coord4i & operator = ( Coord4i const & v);
    inline Coord4i & Set( const int X, const int Y, const int Z, const int W );
    inline Coord4i & Get( int & X, int & Y, int & Z, int & W );
    inline int & operator [] ( const int i );
    inline Coord4i operator + ( Coord4i const & v) const;
    inline Coord4i operator - ( Coord4i const & v) const;
    inline Coord4i operator * ( const int s ) const;
    inline Coord4i operator / ( const int s ) const;
    inline int Norm2( void ) const;
    inline int operator * ( Coord4i const & v ) const;      // Dot product
    inline Coord4i & operator += ( Coord4i const & v);
    inline Coord4i & operator -= ( Coord4i const & v);
    inline Coord4i & operator *= ( const int s );
    inline Coord4i & operator /= ( const int s );
    inline Coord4i & Scale4( const int sx, const int sy,
		                     const int sz, const int sw );
    inline bool operator == ( Coord4i const & v ) const ;
    inline bool operator != ( Coord4i const & v ) const ;
    inline bool operator <  ( Coord4i const & v ) const ;
    inline bool operator >  ( Coord4i const & v ) const ;
    inline bool operator <= ( Coord4i const & v ) const ;
    inline bool operator >= ( Coord4i const & v ) const ;
    inline int qsortcomp( Coord4i const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline int Hash( void ) const;			// range [0-1)
    inline int Dist( Coord4i const & p ) const;
};


inline Coord4i	operator - ( Coord4i const & v );
inline Coord4i	operator * ( const int s, Coord4i const & v );
inline int Norm2( Coord4i const & v );


inline Coord4i::Coord4i ( const int nx, const int ny,
						  const int nz, const int nw )
{
    x = nx;
    y = ny;
    z = nz;
	w = nw;
}


inline Coord4i::Coord4i( Coord4i const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
}


inline Coord4i & Coord4i::operator =( Coord4i const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
    return *this;
}


inline Coord4i & Coord4i::Set( const int X, const int Y,
							   const int Z, const int W )
{
    x= X;
    y= Y;
    z= Z;
	w= W;
    return *this;
} 


inline Coord4i & Coord4i::Get( int & X, int & Y, int & Z, int & W )
{
    X= x;
    Y= y;
    Z= z;
	W= w;
    return *this;
}


inline int & Coord4i::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
		case 3 : return w;
    }
    return x;
}


inline Coord4i Coord4i::operator + ( Coord4i const & v) const
{
    return Coord4i( x+v.x, y+v.y, z+v.z, w+v.w );
}


inline Coord4i Coord4i::operator - ( Coord4i const & v) const
{
    return Coord4i( x-v.x, y-v.y, z-v.z, w-v.w );
}


inline Coord4i Coord4i::operator * ( const int s ) const
{
    return Coord4i( x*s, y*s, z*s, w*s );
}


inline Coord4i Coord4i::operator / ( const int s ) const
{
    return Coord4i( x / s, y / s, z / s, w / s );
} 


inline int Coord4i::operator * ( Coord4i const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z +w*v.w );
}


inline int Coord4i::Norm2( void ) const
{
    return ( x*x + y*y + z*z *w*w );
}


inline Coord4i & Coord4i::operator += ( Coord4i const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
	w += v.w;
    return *this;
}


inline Coord4i & Coord4i::operator -= ( Coord4i const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
	w -= v.w;
    return *this;
}

inline Coord4i & Coord4i::operator *= ( const int s )
{
    x *= s;
    y *= s;
    z *= s;
	w *= s;
    return *this;
}


inline Coord4i & Coord4i::operator /= ( const int s )
{
    x /= s;
    y /= s;
    z /= s;
	w /= s;
    return *this;
}   


inline Coord4i & Coord4i::Scale4( const int sx, const int sy,
								  const int sz, const int sw )
{
    x /= sx;
    y /= sy;
    z /= sz;
	w /= sw;
    return *this;
}


inline bool Coord4i::operator == ( Coord4i const & v ) const
{
    return x==v.x && y==v.y && z==v.z && w==v.w;
}   


inline bool Coord4i::operator != ( Coord4i const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z || w!=v.w;
}


inline bool Coord4i::operator <  ( Coord4i const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z!=v.z)?(z<v.z):
		            (w<v.w);
}


inline bool Coord4i::operator >  ( Coord4i const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z!=v.z)?(z>v.z):
		            (w>v.w);
}


inline bool Coord4i::operator <= ( Coord4i const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z!=v.z)?(z< v.z):
					(w<=v.w);
}


inline bool Coord4i::operator >= ( Coord4i const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z!=v.z)?(z> v.z):
					(w>=v.w);
}


inline int Coord4i::qsortcomp( Coord4i const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
		   (w!=v.w)?( (w>v.w)?1:-1 ):
	       0;
}


inline int Coord4i::Read ( FILE *fp )
{
	return fscanf(fp,"%d%d%d%d",&x,&y,&z,&w);
}


inline int Coord4i::Write( FILE *fp ) const
{
    return fprintf(fp,"%d %d %d %d ",x,y,z,w);
}    


inline int Coord4i::ReadBin( FILE *fp ) {
    int r = 1;
    
    r &= fread(&x,sizeof(int),1,fp);
    r &= fread(&y,sizeof(int),1,fp);
    r &= fread(&z,sizeof(int),1,fp);
	r &= fread(&w,sizeof(int),1,fp);
    return r;
}


inline int Coord4i::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(int),1,fp);
    r &= fwrite(&y,sizeof(int),1,fp);
    r &= fwrite(&z,sizeof(int),1,fp);
	r &= fwrite(&w,sizeof(int),1,fp);
    return r;
}


inline Coord4i operator - ( Coord4i const & v ){
    return Coord4i( -v.x, -v.y, -v.z, -v.w );
}


inline Coord4i operator * ( const int s, Coord4i const & v )
{
    return Coord4i( v.x * s, v.y * s, v.z * s, v.w * s );
}


inline int Norm2( Coord4i const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
}


//}	// End namespace


#endif
