

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_FOG
#define GR_SHQN_FOG


#include "base.h"
#include "node.h"
#include "color.h"

	/** Fog specification.
	    This node manipulates the fog effect.
		@see opengl.
	  */
class grFog : public grNode
{
public:
		/// Default constructor
	inline grFog();
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// see grnode.
	const char * Name() const;
	void Render( grStatus & status );
	
		/// Enable or disable the fog.
	GLboolean  enabled;
		/// Fog blend factor, values: GL_LINEAR, GL_EXP, and GL_EXP2
	GLint	mode;
		/// Fog median density, values: (0 - infinity)
	GLfloat	density;
		/// Depth start position (0,infinity)
	GLfloat start;
		/// Depth end position (>start) (0,infinity)
	GLfloat end;
		/// The Fog color, note: must be equal to background color!
	grColor4f color;

private:
	static const char * name;
};


inline grFog::grFog()
{
	enabled = false;
	mode	= GL_EXP;
	density = 1.0f;
	start	= 0.0f;
	end		= 1.0f;
	color.Set(0.0f,0.0f,0.0f,0.0f);
}


inline void grFog::GL() const
{
	if(enabled)
	{
		glEnable( GL_FOG );
		glFog(GL_FOG_MODE,mode);
		glFog(GL_FOG_DENSITY ,density);
		glFog(GL_FOG_START,start);
		glFog(GL_FOG_END,end);
		glFog(GL_FOG_COLOR,color.color);
	}
	else
	{
		glDisable( GL_FOG );
	}
}


inline void grFog::Get()
{
	if( glIsEnabled( GL_FOG) )
		enabled = true;
	else
		enabled = false;
	glGet(GL_FOG_MODE,&mode);
	glGet(GL_FOG_DENSITY ,&density);
	glGet(GL_FOG_START,&start);
	glGet(GL_FOG_END,&end);
	glGet(GL_FOG_COLOR,color.color);
}


#endif