
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

#pragma warning( disable : 4514 4201 )

#include <windows.h>
#include <stdio.h>
#include <gl/gl.h>
#include <gl/glu.h>


#include "base.h"
#include "gcrwin32.h"


	// Default render values

#define FONT_BASE    1024		// Font-base list name


static BYTE threeto8[8]=  { 0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377 };
static BYTE twoto8[4]  =  { 0, 0x55, 0xaa, 0xff };
static BYTE oneto8[2]  =  { 0, 255 };
static int  defaultOverride[13] = { 0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91 };


static PALETTEENTRY defaultPalEntry[20] = 
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};


static unsigned char ComponentFromIndex(int i, UINT nbits, UINT shift);
static BOOL SetupPixelFormat(HDC hdc);
static BOOL CreateRGBPalette( void );
static void InitializeGL( void );


static unsigned char ComponentFromIndex(int i, UINT nbits, UINT shift)
{
    unsigned char val;

    val = (unsigned char) (i >> shift);
    switch (nbits) 
	{
    case 1: val &= 0x1; return oneto8[val];
    case 2: val &= 0x3; return twoto8[val];
    case 3: val &= 0x7; return threeto8[val];
    default: return 0;
    }
}


static BOOL SetupPixelFormat(HDC hdc) 
{ 
	PIXELFORMATDESCRIPTOR pfd;
    int iPixelFormat;

		// Get current pixel format number.
	iPixelFormat = GetPixelFormat(hdc);

		// Fill in the structure with current information based on number.
	DescribePixelFormat(hdc, iPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

		// Change some p_pfdDescriptor options.
	pfd.dwFlags = 
		PFD_DRAW_TO_WINDOW |            // support window
        PFD_GENERIC_ACCELERATED |
		PFD_SWAP_EXCHANGE |
		PFD_SUPPORT_OPENGL |            // support OpenGL
        PFD_DOUBLEBUFFER;
	pfd.cDepthBits = 32;				// 32 bit color depth.
	pfd.iLayerType = PFD_MAIN_PLANE;

		// get the best available match of pixel format for the device context  
	iPixelFormat = ChoosePixelFormat(hdc, &pfd); 
	if(!iPixelFormat)
	{
		grShowError("Error in ChoosePixelForma");
		return FALSE;
	}
		
		// make that the pixel format of the device context 
	if(!SetPixelFormat(hdc, iPixelFormat, &pfd))
	{
		grShowError("Error in SetPixelFormat");
		return FALSE;
	}

	return TRUE;  
} 


static BOOL CreateRGBPalette( grWindExtra & we )
{
    PIXELFORMATDESCRIPTOR pfd;
    LOGPALETTE *pPal;
    int n, i;
 
    n = ::GetPixelFormat(we.ghDC);
	if(n==0)
	{
		grShowError("Error in GetPixelFormat");
		return FALSE;
	}
    if( !::DescribePixelFormat(we.ghDC, n, sizeof(pfd), &pfd) )
	{
		grShowError("Error in DescribePixelFormat");
		return FALSE;
	}

    if (pfd.dwFlags & PFD_NEED_PALETTE)
    {
        n = 1 << pfd.cColorBits;
        pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];
		if(pPal==0)
		{
			grShowError("Out of memory");
			return FALSE;
		}

        pPal->palVersion = 0x300;
        pPal->palNumEntries = (unsigned short) n;
        for (i=0; i<n; ++i)
        {
            pPal->palPalEntry[i].peRed =
                    ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
            pPal->palPalEntry[i].peGreen =
                    ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
            pPal->palPalEntry[i].peBlue =
                    ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
            pPal->palPalEntry[i].peFlags = 0;
        }

			// fix up the palette to include the default GDI palette
        if ((pfd.cColorBits == 8)                           &&
            (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
            (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
            (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
           )
        {
			for (i = 1 ; i <= 12 ; ++i)
                pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
        }

        we.hPale = ::CreatePalette(pPal);
		if(we.hPale==0)
		{
			grShowError("Error in CreatePalette");
			return FALSE;
		}
        delete[] pPal;

        we.hOldP = ::SelectPalette(we.ghDC,we.hPale, FALSE);
		if(we.hOldP==0)
		{
			grShowError("Error in SelectPalette");
			return FALSE;
		}

        if( RealizePalette(we.ghDC)==GDI_ERROR )
		{
			grShowError("Error in RealizePalette");
			return FALSE;
		}
    }
	return TRUE;
}


static void InitializeGL( void ) 
{ 
		// Colori

	const GLclampf BKColor[4]		={ 0.0f, 0.0f, 0.5f, 1.0f };
	const GLfloat FogColor[4]		={ 1.0f, 1.0f, 1.0f, 1.0f };
	const GLfloat AmbientColor[4]	={ 0.2f, 0.2f, 0.2f, 1.0f };

		// Settaggi Generali

	glRenderMode( GL_RENDER );
	glClearColor(BKColor[0],BKColor[1],BKColor[2],BKColor[3]);
	glClearDepth( GLclampd(1.0) );
	glShadeModel( GL_SMOOTH );
	glCullFace( GL_BACK );
	
		// Abilitazioni/Disabilitazioni

	glDisable( GL_ALPHA_TEST );
	glDisable( GL_AUTO_NORMAL );
	glDisable( GL_BLEND );
	glDisable( GL_COLOR_ARRAY_EXT );
	glDisable( GL_COLOR_MATERIAL );
	glDisable( GL_CULL_FACE );
	glEnable ( GL_DEPTH_TEST ); 
	glDisable( GL_DITHER ); 
	glDisable( GL_EDGE_FLAG_ARRAY_EXT );
	//glEnable ( GL_FOG );
	glDisable( GL_INDEX_ARRAY_EXT );
	glDisable( GL_LINE_SMOOTH );
	glEnable ( GL_LINE_STIPPLE );
	glDisable( GL_LOGIC_OP );
	glDisable( GL_NORMAL_ARRAY_EXT );
	glEnable ( GL_NORMALIZE );
	glDisable( GL_POINT_SMOOTH );
	glDisable( GL_POLYGON_SMOOTH );
	glDisable( GL_POLYGON_STIPPLE );
	glDisable( GL_SCISSOR_TEST );
	glDisable( GL_STENCIL_TEST );

	glPixelStorei( GL_PACK_ROW_LENGTH, 0);
	glPixelStorei( GL_PACK_ALIGNMENT, 1);

	//glLineStipple(4,0x5555 );

		// Parametri illuminazione

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
    glEnable(GL_LIGHTING);

	glLightModeli (GL_LIGHT_MODEL_LOCAL_VIEWER,0);
	glLightModeli (GL_LIGHT_MODEL_TWO_SIDE,    0);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, AmbientColor);

	float fv4[4];
    int iv1[1];

	glLoadIdentity();
	fv4[0] = 0.0f;
    fv4[1] = 0.0f;
    fv4[2] = 1.0f;
    fv4[3] = 0.0f;
    glLightfv(GL_LIGHT0, GL_POSITION, fv4);	
	fv4[0] =  0.0f;
    fv4[1] =  0.0f;
    fv4[2] = -1.0f;
    fv4[3] =  0.0f;
    glLightfv(GL_LIGHT1, GL_POSITION, fv4);	
	fv4[0] = 0.9f;
    fv4[1] = 0.9f;
    fv4[2] = 0.9f;
    fv4[3] = 1.0f;
    glLightfv(GL_LIGHT0, GL_DIFFUSE, fv4);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, fv4);

		// Definizione colore base
	fv4[0] = 0.1f;
    fv4[1] = 0.1f;
    fv4[2] = 0.1f;
    fv4[3] = 1.0f;
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, fv4);
    fv4[0] = 0.0f;
    fv4[1] = 0.0f;
    fv4[2] = 0.0f;
    fv4[3] = 1.0f;
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fv4);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, fv4);
    iv1[0] = 0;
    glMaterialiv(GL_FRONT_AND_BACK, GL_SHININESS, iv1);
	fv4[0] = 0.9f;
    fv4[1] = 0.9f;
    fv4[2] = 0.9f;
    fv4[3] = 0.9f;
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, fv4 );
	
}


void grShowError( const char * msg )
{
	LPVOID lpMsgBuf;
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		0,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR) &lpMsgBuf,
		0,
		0
	);

	if(lpMsgBuf!=0)
	{
		MessageBox(0,(char *)lpMsgBuf,msg,MB_OK|MB_ICONERROR);
		LocalFree( lpMsgBuf );
	}
}


void grGetGLInfo( char * str )
{
	strcpy(str,"Vendor:\t ");
	strcat(str,(const char *)glGetString(GL_VENDOR));
	strcat(str,"\nRender:\t ");
	strcat(str,(const char *)glGetString(GL_RENDERER));
	strcat(str,"\nVersion:\t ");
	strcat(str,(const char *)glGetString(GL_VERSION));

	char *p;
	const char *t;

	strcat(str,"\nExtensions:\n\t");
	t = (const char *)glGetString(GL_EXTENSIONS);
	p = str+strlen(str);
	while(*t)
	{
		if(*t==' ')
		{
			*p++ = '\n';
			*p++ = '\t';
			++t;
		}
		else
		{
			*p++ = *t++;
		}
	}
	*p = 0;

	strcat(str,"\n\nGLU Version:\t ");
	strcat(str,(const char *)gluGetString(GLU_VERSION));

	strcat(str,"\nGLU Extensions:\n\t");
	t = (const char *)gluGetString(GLU_EXTENSIONS);
	p = str+strlen(str);
	while(*t)
	{
		if(*t==' ')
		{
			*p++ = '\n';
			*p++ = '\t';
			++t;
		}
		else
		{
			*p++ = *t++;
		}
	}
	*p = 0;

	strcat(str,"\n\nGCR Version:\t 2.1 (C)1999 VCGroup\n");
}


BOOL grOnCreate( HWND hwnd, grWindExtra & we )
{
	we.ghDC = GetDC(hwnd);
	if(we.ghDC==0)
	{
		grShowError("Error in GetDC");
		return FALSE;
	}


    if( !SetupPixelFormat(we.ghDC) )
	{
		::ReleaseDC(hwnd, we.ghDC);
		return FALSE;
	}
	if( !CreateRGBPalette(we) )
	{
		::ReleaseDC(hwnd, we.ghDC);
		return FALSE;
	}

	we.ghRC = wglCreateContext(we.ghDC);
	if(we.ghRC==0)
	{
		grShowError("Error in wglCreateContext");
		if (we.hOldP!=0)
		{
			::SelectPalette(we.ghDC, we.hOldP, FALSE);
			::DeleteObject(we.hPale);
			we.hPale = 0;
			we.hOldP = 0;
		}
		::ReleaseDC(hwnd, we.ghDC);
		we.ghDC = 0;
		return FALSE;
	}

    if(!wglMakeCurrent(we.ghDC, we.ghRC))
	{
		grShowError("Error in wglMakeCurrent");
		if (we.ghRC!=0)
		{
			::wglDeleteContext(we.ghRC);
			we.ghRC = 0;
		}
		if (we.hOldP!=0)
		{
			::SelectPalette(we.ghDC, we.hOldP, FALSE);
			::DeleteObject(we.hPale);
			we.hPale = 0;
			we.hOldP = 0;
		}
		::ReleaseDC(hwnd, we.ghDC);
		we.ghDC = 0;
		return FALSE;
	}

	SelectObject(we.ghDC, GetStockObject (SYSTEM_FONT));
	if( !wglUseFontBitmaps(we.ghDC, 0, 256, FONT_BASE) )
	{
		grShowError("Error in wglUseFontBitmaps");
		::wglMakeCurrent(0,  0);
		if (we.ghRC!=0)
		{
			::wglDeleteContext(we.ghRC);
			we.ghRC = 0;
		}
		if (we.hOldP!=0)
		{
			::SelectPalette(we.ghDC, we.hOldP, FALSE);
			::DeleteObject(we.hPale);
			we.hPale = 0;
			we.hOldP = 0;
		}
		::ReleaseDC(hwnd, we.ghDC);
		we.ghDC = 0;
		return FALSE;
	}
	
		// Inizializzazione dati di default
	we.bCapture  = FALSE;
	we.aspect	 = 0.0f;
	we.nWidth	 = 0;
	we.nHeight	 = 0;
	we.nearPlane = 0.0001;
	we.farPlane  = 30.0;
	we.prospAngle= 45.0;

	InitializeGL();

	return TRUE;
}


void grOnDestroy( HWND hwnd, grWindExtra & we )
{
	if(we.bCapture)
	{
		::ReleaseCapture();
		we.bCapture = FALSE;
	}

	glFinish();
	glDeleteLists(FONT_BASE, 256);

	::wglMakeCurrent(0,  0);

    if (we.ghRC!=0)
	{
		::wglDeleteContext(we.ghRC);
		we.ghRC = 0;
	}

	if (we.hOldP!=0)
	{
		::SelectPalette(we.ghDC, we.hOldP, FALSE);
		::DeleteObject(we.hPale);
		we.hPale = 0;
		we.hOldP = 0;
	}

    if(we.ghDC)
	{
		::ReleaseDC(hwnd, we.ghDC);
		we.ghDC = 0;
	}
}


void grOnSize( HWND /*hwnd*/, grWindExtra & we, int w, int h )
{
	we.nWidth	= w;
	we.nHeight	= h;
	glViewport( 0, 0, w, h);

	if(w!=0)
	{
		we.aspect = GLfloat(w)/h;
		glMatrixMode( GL_PROJECTION ); 
		glLoadIdentity(); 
		gluPerspective( we.prospAngle, we.aspect, we.nearPlane, we.farPlane );
		glMatrixMode( GL_MODELVIEW );

		glGet(GL_VIEWPORT,we.viewport);
		glGet(GL_PROJECTION_MATRIX,we.projMatrix);
	}
}

