

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_LIGHT
#define GR_SHQN_LIGHT


#include "base.h"
#include "node.h"
#include "color.h"
#include "coord3f.h"
#include "coord4f.h"

	/** Light definition.
	    This node defines a new user light. See opengl references for the
		definition of lights parameters. The function SetZero link the object to
		the light0 and support the manipulation of the standard opengl light.
		NOTA: questo nodo e' in versione BETA.
		@see opengl.
	 */
class grLight : public grNode
{
public:
		/// Default constructor
	inline  grLight();
		/// Default distructor
	inline ~grLight();
	inline void GL() const;
		/// Retrieves settings from internal opengl status, relative of this light
	inline void Get();
		/// Links this object to the opengl light0. The opengl light0 is the standard opengl light.
	inline void SetZero();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	
	static inline int  AllocLight( void );
	static inline void FreeLight( const int l );

		/// Light spot exponent, values: (0-128)
	GLfloat SpotExponent;
		/// Light spot cut off, values: 128 or (0-90)
	GLfloat SpotCutoff;
		/// Light constant attenuation, values: (0, infinity)
	GLfloat CAttenuation;
		/// Light linear attenuation, values: (0, infinity)
	GLfloat LAttenuation;
		/// Light quadratic attenuation, values: (0, infinity)
	GLfloat QAttenuation;
		/// Light ambient color
	grColor4f Ambient;
		/// Light diffuse color
	grColor4f Diffuse;
		/// Light specular color
	grColor4f Specular;
		/// Light position, (may be an infinite point)
	Coord4f   Position;
		/// Light direction
	Coord3f	  SpotDirection;

private:
	GLenum		light;
	
	static BOOL lights[GL_MAX_LIGHTS];
	static const char * name;
};


inline int grLight::AllocLight()
{
	int i;
	for(i=1;lights[i];++i)
		;
	lights[i] = TRUE;
	return i;
}


inline void grLight::FreeLight( const int l )
{
	assert(l>=1);
	assert(l<GL_MAX_LIGHTS);
	assert(lights[l]);
	lights[l] = FALSE;
}


inline grLight::grLight()
{
	SpotExponent	= 0.0f;
	SpotCutoff		= 180.0f;
	CAttenuation	= 1.0f;
	LAttenuation	= 0.0f;
	QAttenuation	= 0.0f;
	Ambient.Set(0.0f,0.0f,0.0f,1.0f);
	Diffuse.Set(0.0f,0.0f,0.0f,1.0f);
	Specular.Set(0.0f,0.0f,0.0f,1.0f);
	Position.Set(0.0f,0.0f,1.0f,0.0f);
	SpotDirection.Set(0.0f,0.0f,-1.0f);
	light = AllocLight();
}


inline grLight::~grLight()
{
	if(light!=-1)
		FreeLight(light);
}


inline void grLight::GL() const
{
	glLight(light,GL_SPOT_EXPONENT,SpotExponent);
	glLight(light,GL_SPOT_CUTOFF,SpotCutoff);
	glLight(light,GL_CONSTANT_ATTENUATION,CAttenuation);
	glLight(light,GL_LINEAR_ATTENUATION,LAttenuation); 
	glLight(light,GL_QUADRATIC_ATTENUATION,QAttenuation);
	glLight(light,GL_AMBIENT,Ambient.color);
	glLight(light,GL_DIFFUSE,Diffuse.color);
	glLight(light,GL_SPECULAR,Specular.color);

	GLfloat b[4];
	b[0] = Position.x;
	b[1] = Position.y;
	b[2] = Position.z;
	b[3] = Position.w;
	glLight(light,GL_POSITION,b);
	b[0] = SpotDirection.x;
	b[1] = SpotDirection.y;
	b[2] = SpotDirection.z;
	glLight(light,GL_SPOT_DIRECTION,b);

}


inline void grLight::Get()
{
	glGetLight(light,GL_SPOT_EXPONENT,&SpotExponent);
	glGetLight(light,GL_SPOT_CUTOFF,&SpotCutoff);
	glGetLight(light,GL_CONSTANT_ATTENUATION,&CAttenuation);
	glGetLight(light,GL_LINEAR_ATTENUATION,&LAttenuation); 
	glGetLight(light,GL_QUADRATIC_ATTENUATION,&QAttenuation);
	glGetLight(light,GL_AMBIENT,Ambient.color);
	glGetLight(light,GL_DIFFUSE,Diffuse.color);
	glGetLight(light,GL_SPECULAR,Specular.color);
	GLfloat b[4];
	glGetLight(light,GL_POSITION,b);
	Position.x = b[0];
	Position.y = b[1];
	Position.z = b[2];
	Position.w = b[3];
	glGetLight(light,GL_SPOT_DIRECTION,b);
	SpotDirection.x = b[0];
	SpotDirection.y = b[1];
	SpotDirection.z = b[2];
}


inline void grLight::SetZero()
{
	if(light!=-1)
		FreeLight(light);
	light = 0;
}


#endif