
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_NORMAL
#define GR_SHQN_NORMAL


#include <vector>
#include "base.h"
#include "node.h"
#include "coord3d.h"
#include "coord3f.h"
#include "coord3i.h"
#include "coord3s.h"
#include "status.h"


using namespace std;

	/** Abstract base class for all normals set nodes.
	    This is the abstract base class from wich all normals node
		classes are derived. A normal node is a 'set' of
		versors (not a single versor).
		@see grPrimitive, STL
	 */
class grNormal : public grNode
{
public:
	void Render( grStatus & status );
		/// The size of the set (pure virtual)
	virtual int Size() const = 0 ;
	virtual void GL( const int i ) const = 0;
};

	/** @name Normal sub-classes.
	  */
	//@{

	/// set of normals (double coordinates)
class grNormald : public grNormal, virtual public vector<Coord3d>
{
public:
		/// see grNode
	const char * Name() const;
		/// see grNormal
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// set of normals (float coordinates)
class grNormalf : public grNormal, virtual public vector<Coord3f>
{
public:
		/// see grNode
	const char * Name() const;
		/// see grNormal
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// set of normals (integer coordinates)
class grNormali : public grNormal, virtual public vector<Coord3i>
{
public:
		/// see grNode
	const char * Name() const;
		/// see grNormal
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// set of normals (short integer coordinates)
class grNormals : public grNormal, virtual public vector<Coord3s>
{
public:
		/// see grNode
	const char * Name() const;
		/// see grNormal
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	//@}

#endif
