
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_PICK
#define GR_SHQN_PICK


#include <vector>
#include "node.h"


using namespace std;

	/** @name Picking system.
	  */
	//@{

	/** Selection results item.
	    This class memorize a Pick-selection item results of the selection traversal.
		This record consists of a set of pointers to the nodes selected
		and a interval by the minimum and maximum depth values of all vertices that hit.
	    @see grRender, grPickResult
	 */
class grHitRecord
{
public:
		/// Miminum depth value of the selection interval
	double minDepth;
		/// Maxinum depth value of the selection interval
	double maxDepth;
		/// Set of node pointers selected
	vector<grNode * > nameStack;

	inline bool operator <  ( const grHitRecord & r ) const;
	inline bool operator == ( const grHitRecord & r ) const;
};

	/** @name grPickResult
		Selection results.
		This class memorize a Pick-selection results of the selection traversal.
		The result is a set of Hit Records, ordered by depth:
		(vector<grHitRecord>  = grPickResult).
		@see grHitRecord, grRender
	 */
typedef vector<grHitRecord> grPickResult;


inline bool grHitRecord::operator <  ( const grHitRecord & r ) const
{
	return minDepth < r.minDepth;
}


inline bool grHitRecord::operator == ( const grHitRecord & r ) const
{
	return minDepth == r.minDepth;
}

	//@}

#endif