
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_SCISSOR
#define GR_SHQN_SCISSOR


#include "base.h"
#include "node.h"


	/** Scissor box definition.
		The grScissor defines a rectangle, called the scissor box, in window coordinates.
		The first two parameters, x and y, specify the lower-left corner of the box.
		The width and height parameters specify the width and height of the box.
		The scissor box may be enabled or disabled.
		@see opengl
	  */
class grScissor : public grNode
{
public:
		/// Default constructor
	inline grScissor();
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	
		/// Scissor x start position, in window coordinate.
	GLint x;
		/// Scissor y start position, in window coordinate.
	GLint y;
		/// Scissor width.
	GLsizei width;
		/// Scissor height.
	GLsizei height;
		/// Scissor enabling/disabling flag.
	GLboolean enabled;

private:
	static const char * name;
};


inline grScissor::grScissor()
{
	x = 0;
	y = 0;
	width  = 0;
	height = 0;
	enabled = false;
}


inline void grScissor::GL() const
{
	if(enabled)
	{
		glEnable(GL_SCISSOR_TEST);
		glScissor(x,y,width,height);
	}
	else
	{
		glDisable(GL_SCISSOR_TEST);
	}
}


inline void grScissor::Get()
{
	int t[4];
	glGet(GL_SCISSOR_BOX ,t);
	x = t[0];
	y = t[1];
	width  = t[2];
	height = t[3];

	enabled = glIsEnabled(GL_SCISSOR_TEST);
}



#endif
