
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_SPECS
#define GR_SHQN_SPECS


#include "base.h"
#include "node.h"


	/** General rendering settings.
	    This node specifies a set of general rendering setting:
		the shade model (GL_FLAT = faces are flat, GL_SMOOTH), the
		front face definition (GL_CCW = the front is the counter clock-wise
		order, GL_CW = the front is the clock-wise order) and the cull-face
		method (GL_FRONT = the front is culled, GL_BACK = the back is culled).
	    @see opengl
	 */
class grSpecs : public grNode
{
public:
		/// Default constructor
	inline grSpecs();
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );

		/// Face culling method: GL_FRONT or GL_BACK
	GLenum  CullFace;
		/// Front face definition: GL_CCW or GL_CW	
	GLenum  FrontFace;
		/// Face shading model: GL_FLAT or GL_SMOOTH
	GLenum  ShadeModel;

	GLfloat LineWidth;
	GLfloat PointSize;

private:
	static const char * name;
};


inline grSpecs::grSpecs()
{
	CullFace	= GL_BACK;
	FrontFace	= GL_CCW;
	ShadeModel  = GL_SMOOTH;
	LineWidth	= 1.0f;
	PointSize	= 1.0f;
}


inline void grSpecs::GL() const
{
	glCullFace(CullFace);
	glFrontFace(FrontFace);
	glShadeModel(ShadeModel);
	glLineWidth(LineWidth);
	glPointSize(PointSize);}


inline void grSpecs::Get()
{
	glGet(GL_CULL_FACE_MODE,(int *)&CullFace);
	glGet(GL_FRONT_FACE,(int *)&FrontFace);
	glGet(GL_SHADE_MODEL ,(int *)&ShadeModel);
	glGet(GL_LINE_WIDTH,&LineWidth);
	glGet(GL_POINT_SIZE,&PointSize);
}


#endif