

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_SPHERE
#define GR_SHQN_SPHERE


#include "quadric.h"

	/** Sphere shape node.
	    The sphere function draws a sphere of the given radius centered around the origin.
		The sphere is subdivided around the z-axis into slices and along
		the z-axis into stacks (similar to lines of longitude and latitude).
		@see GLU
	 */
class grSphere : public grQuadric
{
public:
		/// Standard constructor
	inline  grSphere();
		/// Constructor by radius
	inline  grSphere( const double radius );
	inline void GL( ) const;

		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	void Select( grStatus & status );
	
		/// Radius of sphere
	double radius;
		/// Number of meridians (default 16)
	int    slices;
		/// Number of parallels (default 16)
	int    stacks;

private:
	static const char * name;
};


inline grSphere::grSphere()
{
	radius = 1.0;
	slices = 16;
	stacks = 16;
}


inline grSphere::grSphere( const double r )
{
	radius = r;
	slices = 16;
	stacks = 16;
}


inline void grSphere::GL() const
{
	gluSphere( q,radius,slices,stacks );
}



#endif