
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_TEXCOORD
#define GR_SHQN_TEXCOORD


#include <vector>
#include "base.h"
#include "node.h"
#include "status.h"
#include "coord.h"


using namespace std;

	/** Abstract base class for all texture coordinate nodes.
	    This is the abstract base class from wich all 
		texture coordinates node are derived.
		@see opengl
	 */
class grTexCoord : public grNode
{
public:
		/// Number of texture coordinates (pure virtual)
	virtual int Size(  ) const = 0;
	virtual void GL( const int i ) const = 0;

	void Render( grStatus & status );
};

	/** @name Texture coordinates sub-classes
	  */
	//@{

	/** Texture coordinates.
	    Set of 1-dimensional texture coordinate (double).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord1d : public grTexCoord , public vector<double> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 1-dimensional texture coordinate (float).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord1f : public grTexCoord , public vector<float> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 1-dimensional texture coordinate (int).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord1i : public grTexCoord , public vector<int> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 1-dimensional texture coordinate (short).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord1s : public grTexCoord , public vector<short> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 2-dimensional texture coordinate (double).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord2d : public grTexCoord , public vector<Coord2d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 2-dimensional texture coordinate (float).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord2f : public grTexCoord , public vector<Coord2f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 2-dimensional texture coordinate (int).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord2i : public grTexCoord , public vector<Coord2i> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 2-dimensional texture coordinate (short).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord2s : public grTexCoord , public vector<Coord2s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 3-dimensional texture coordinate (double).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord3d : public grTexCoord , public vector<Coord3d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 3-dimensional texture coordinate (float).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord3f : public grTexCoord , public vector<Coord3f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 3-dimensional texture coordinate (int).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord3i : public grTexCoord , public vector<Coord3i> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 3-dimensional texture coordinate (short).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord3s : public grTexCoord , public vector<Coord3s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};


	/** Texture coordinates.
	    Set of 4-dimensional texture coordinate (double).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord4d : public grTexCoord , public vector<Coord4d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 4-dimensional texture coordinate (float).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord4f : public grTexCoord , public vector<Coord4f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 4-dimensional texture coordinate (int).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord4i : public grTexCoord , public vector<Coord4i> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/** Texture coordinates.
	    Set of 4-dimensional texture coordinate (short).
		@see STL, opengl, grTexCoord.
	 */
class grTexCoord4s : public grTexCoord , public vector<Coord4s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grTexCoord
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	//@}

#endif
