

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_TEXTURE
#define GR_SHQN_TEXTURE


#include "base.h"
#include "node.h"
#include "color.h"

	/** Texture Image specification.
	    This node specifies the current texture image.
		@see opengl, grTexCoord.
	 */
class grTexture : public grNode
{
public:
		/// Default Constructor
	inline grTexture();
	inline void GL() const;
		/// Retrieve the texture settings from the opengl internal status
	inline void Get();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	
		/// Sub-pixel filter mode, values: GL_NEAREST, GL_LINEAR, GL_NEAREST_MIPMAP_NEAREST, GL_LINEAR_MIPMAP_NEAREST, GL_NEAREST_MIPMAP_LINEAR, GL_LINEAR_MIPMAP_LINEAR
	GLint minfilter;
		/// Multi-pixel filter mode, values: GL_NEAREST or GL_LINEAR.
	GLint magfilter;
		/// Image wrapping mode for s coordinate, values: GL_CLAMP or GL_REPEAT
	GLint wraps;
		/// Image wrapping mode for t coordinate, values: GL_CLAMP or GL_REPEAT
	GLint wrapt;
		/// Color of the image border
	grColor4f bordercolor;
		/// Number of component of the image: 1,2,3,4, (ie rgb=3, rgba=4)
	GLint components;
		/// Width of the texture image, must be a power of 2 (+1 if border is present).
	GLsizei width;
		/// Height of the texture image, must be a power of 2 (+1 if border is present).
	GLsizei height;
		/// Must be either 0 (no border) or 1.
	GLsizei border;
		/// Image pixel format, values: GL_COLOR_INDEX,GL_RED,GL_GREEN,GL_BLUE,GL_ALPHA,GL_RGB, GL_RGBA,GL_LUMINANCE
	GLenum  format;
		/// Image pixel type, values: GL_UNSIGNED_BYTE, GL_BYTE, GL_BITMAP, GL_UNSIGNED_SHORT, GL_SHORT, GL_UNSIGNED_INT, GL_INT, and GL_FLOAT
	GLenum  type;
		/// Pointer of picture data.
	void * memory;

private:
	static const char * name;
};


inline grTexture::grTexture()
{
	minfilter = GL_NEAREST_MIPMAP_LINEAR;
	magfilter = GL_LINEAR;
	wraps     = GL_REPEAT;
	wrapt     = GL_REPEAT;
	bordercolor.Set(0,0,0,0);

	components = 4;
	width  = 0; 
	height = 0;
	border = 0;
	format = 0;
	type   = 0;
	memory = 0;
}


inline void grTexture::GL() const
{
	glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minfilter);
	glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magfilter);
	glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,wraps);
	glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,wrapt);
	glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_BORDER_COLOR,bordercolor.color);

	glTexImage2D(GL_TEXTURE_2D,0,components,width,height,border,format,type,memory);
}


inline void grTexture::Get()
{
	glGetTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,&minfilter);
	glGetTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,&magfilter);
	glGetTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,&wraps);
	glGetTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,&wrapt);
	glGetTexParameter(GL_TEXTURE_2D,GL_TEXTURE_BORDER_COLOR,bordercolor.color);
}


#endif