
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

#pragma warning( disable : 4514 4201 )

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <assert.h>

#include "status.h"
#include "tmesh.h"
#include "depth.h"


static void BoxGL( const Boxf & b );


const char * grTMeshViewMode::name = "TMeshViewMode";


const char * grTMeshViewMode::Name() const
{
	return name;
}


void grTMeshViewMode::Render( grStatus & status )
{
	status.tmvmode = vmode;
}



const char * grTMesh::name = "TMesh";


const char * grTMesh::Name() const
{
	return name;
}


void grTMesh::Select( grStatus & status )
{
	if(status.onesel==0 || status.onesel!=this)
	{
		if(status.onesel==0)
			glPushName(glname);

		Render(status);

		if(status.onesel==0)
			glPopName();
	}
	else
	{
		bool cont = true;

		switch(status.tmvmode)
		{
		case grTMeshViewMode::WIRE_AND_SHADE:
		case grTMeshViewMode::FLAT_SHADED:
			StartScanFace();
			do
			{
				glPushName(CurrentFaceName());
				glBegin(GL_TRIANGLES);
				cont = NextFace(status);
				glEnd();
				glPopName();
			} while(cont);
			break;

		case grTMeshViewMode::HIDDEN_LINE:
		case grTMeshViewMode::WIRE_FRAME:
			StartScanEdge();
			do
			{
				glPushName(CurrentEdgeName());
				glBegin(GL_LINES);
					cont = NextEdge(status);
				glEnd();
				glPopName();
			} while(cont);
			break;

		case grTMeshViewMode::POINTS:
			StartScanVert();
			do
			{
				glPushName(CurrentVertName());
				glBegin(GL_POINTS);
					cont = NextVert(status);
				glEnd();
				glPopName();
			} while(cont);
			break;

		case grTMeshViewMode::BOUND_BOX:
			break;
		}
	}
}

grNode * grTMesh::FindByName( const GLuint name )
{
	if(name==glname)
		return this;
	else
		return 0;
}

void grTMesh::Render( grStatus & status )
{
	const float ZTIWST =  0.001f;
	grDepth tmp;

	switch(status.tmvmode)
	{
	case grTMeshViewMode::FLAT_SHADED:
		StartScanFace();
		glBegin(GL_TRIANGLES);
			while( NextFace(status) )
				;
		glEnd();
		break;

	case grTMeshViewMode::WIRE_FRAME:
		StartScanEdge();
		glBegin(GL_LINES);
			while( NextEdge(status) )
				;
		glEnd();
		break;
	
	case grTMeshViewMode::HIDDEN_LINE:
		tmp.Get();
		glDepthRange(ZTIWST,1.0f);
		glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);
		StartScanFace();
		glBegin(GL_TRIANGLES);
			while( NextFace(status) )
				;
		glEnd();
		glDepthRange(0.0f,1.0f-ZTIWST);
		glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
		glDepthFunc(GL_LEQUAL);
		StartScanEdge();
		glBegin(GL_LINES);
			while( NextEdge(status) )
				;
		glEnd();
		tmp.GL();
		break;

	case grTMeshViewMode::WIRE_AND_SHADE:
		tmp.Get();
		glDepthRange(ZTIWST,1.0f);
		StartScanFace();
		glBegin(GL_TRIANGLES);
			while( NextFace(status) )
				;
		glEnd();
		glDepthRange(0.0f,1.0f-ZTIWST);
		glDepthFunc(GL_LEQUAL);
		StartScanEdge();
		glBegin(GL_LINES);
			while( NextEdge(status) )
				;
		glEnd();
		tmp.GL();
		break;

	case grTMeshViewMode::POINTS:
		StartScanVert();
		glBegin(GL_POINTS);
			while( NextVert(status) )
				;
		glEnd();
		break;

	case grTMeshViewMode::BOUND_BOX:
		BoxGL(BoundBox());
		break;

	default:
		break;

	}
}


static void BoxGL( const Boxf & b )
{
	glBegin(GL_LINES);

	glColor3f(1.0f,1.0f,1.0f);

	glNormal3f(-1.0f,-1.0f,-1.0f);
	glVertex3f( b.min.x, b.min.y, b.min.z );
	glNormal3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( b.min.x, b.min.y, b.max.z );
	glNormal3f(-1.0f,-1.0f,-1.0f);
	glVertex3f( b.min.x, b.min.y, b.min.z );
	glNormal3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( b.min.x, b.max.y, b.min.z );
	glNormal3f(-1.0f,-1.0f,-1.0f);
	glVertex3f( b.min.x, b.min.y, b.min.z );
	glNormal3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( b.max.x, b.min.y, b.min.z );

	glNormal3f( 1.0f, 1.0f, 1.0f);
	glVertex3f( b.max.x, b.max.y, b.max.z );
	glNormal3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( b.max.x, b.max.y, b.min.z );
	glNormal3f( 1.0f, 1.0f, 1.0f);
	glVertex3f( b.max.x, b.max.y, b.max.z );
	glNormal3f( 1.0f,-1.0f, 1.0f);
	glVertex3f( b.max.x, b.min.y, b.max.z );
	glNormal3f( 1.0f, 1.0f, 1.0f);
	glVertex3f( b.max.x, b.max.y, b.max.z );
	glNormal3f(-1.0f, 1.0f, 1.0f);
	glVertex3f( b.min.x, b.max.y, b.max.z );

	glNormal3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( b.min.x, b.min.y, b.max.z );
	glNormal3f( 1.0f,-1.0f, 1.0f);
	glVertex3f( b.max.x, b.min.y, b.max.z );
	glNormal3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( b.min.x, b.min.y, b.max.z );
	glNormal3f(-1.0f, 1.0f, 1.0f);
	glVertex3f( b.min.x, b.max.y, b.max.z );

	glNormal3f( 1.0f,-1.0f,-1.0f);
	glVertex3f(	b.max.x, b.min.y, b.min.z );
	glNormal3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( b.max.x, b.max.y, b.min.z );
	glNormal3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( b.max.x, b.min.y, b.min.z );
	glNormal3f( 1.0f,-1.0f, 1.0f);
	glVertex3f( b.max.x, b.min.y, b.max.z );

	glNormal3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( b.min.x, b.max.y, b.min.z );
	glNormal3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( b.max.x, b.max.y, b.min.z );
	glNormal3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( b.min.x, b.max.y, b.min.z );
	glNormal3f(-1.0f, 1.0f, 1.0f);
	glVertex3f( b.min.x, b.max.y, b.max.z );

	glEnd();
}
