

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_TMESH
#define GR_SHQN_TMESH


#include "base.h"
#include "node.h"
#include "box.h"
#include "status.h"

	/** TMesh rendering mode.
	    This node defines the current TMesh rendering mode; the modes are:
		FLAT_SHADED, HIDDEN_LINE, WIRE_FRAME, WIRE_AND_SHADE, POINTS, BOUND_BOX.
		@see opengl, grTMesh.
	 */
class grTMeshViewMode : public grNode
{
public:
		// Modi di visualizzazione
	enum { FLAT_SHADED, HIDDEN_LINE, WIRE_FRAME, WIRE_AND_SHADE, POINTS, BOUND_BOX };

		/// Default constructor
	inline grTMeshViewMode();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
		
		/// Renderin mode to set
	int vmode;

private:
	static const char * name;
};

	/** User triangular meshes helper.
	    This is the pure abstract base class from wich the user
		may derives a triangular mesh node.
		The functions BoundBox, Start(X), Next(X) are pure virtual
		and the user must defines it in the derived class.
		The functions NextVert must generate a glVertex call, the
		function NextFace 3 glVertex calls and the function NextEdge
		2 glVertex. At the end of scanning this functions must return
		false.
		(??) Le funzioni next ritornano false a fine scansione (cioe'
		l'ultima scansione ritorna true, quella dopo false, insomma
		se ci sono tre vertici, nextvert ritorna 3 volte true e la quarta
		false) (??).
		The function BoundBox must return the bounding box of the surface.
		@see grTMeshViewMode.
	 */
class grTMesh : public grNode
{
public:
		/// Default constructor
	inline grTMesh();
		/// see grNode
	const char * Name( void ) const;
	void Render( grStatus & status );
	void Select( grStatus & status );
	grNode * FindByName( const GLuint name );

		
		// Private investigation member : user MUST define it

		/// Must return the bounding box of the surface.
	virtual const Boxf & BoundBox() = 0;
		/// The system starts a vertex request
	virtual void StartScanVert() = 0;
		/// The system requires a vertex (1 glVertex call)
	virtual bool NextVert( grStatus & status ) = 0;
		/// The system starts a face request
	virtual void StartScanFace() = 0;
		/// The system requires a triangle (3 glVertex calls)
	virtual bool NextFace( grStatus & status ) = 0;
		/// The system starts an edge request
	virtual void StartScanEdge() = 0;
		/// The system requires an edge (2 glVertex calls)
	virtual bool NextEdge( grStatus & status ) = 0;
		/// Must return the current vertex name
	virtual GLuint CurrentVertName() = 0;
		/// Must return the current face vertex
	virtual GLuint CurrentFaceName() = 0;
		/// Must return the current edge vertex
	virtual GLuint CurrentEdgeName() = 0;

private:
	static const char * name;

protected:
	GLuint glname;
};


inline grTMesh::grTMesh()
{ 
	glname = ++grStatus::lastname;
}

inline grTMeshViewMode::grTMeshViewMode()
{
	vmode = FLAT_SHADED;
}


#endif