
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_TRANSFORM
#define GR_SHQN_TRANSFORM


#include "base.h"
#include "node.h"
#include "coord3f.h"
#include "coord3d.h"
#include "matrix.h"


	/** Abstract base class for all transformations nodes.
	    This is the abstract base class from wich all geometric
		trasformation nodes are derived.
		@see opengl.
	 */
class grTransform : public grNode
{
public:
		/// Default constructor
	inline grTransform();

	virtual void GL( void ) const = 0;

	void Render( grStatus & status );
	void Select( grStatus & status );

		/// Select the trasformation mode: GL_MODELVIEW, GL_PROJECTION or GL_TEXTURE
	GLenum mode;	
};

	/// Scene translation: the coordinate represents a translation
class grTranslationf : public grTransform, public Coord3f
{
public:
		/// Default constructor
	inline grTranslationf();
		/// see grNode
	const char * Name() const;
	void GL() const;

private:
	static const char * name;
};

	/// Scene translation: the coordinate represents a translation
class grTranslationd : public grTransform, public Coord3d
{
public:
		/// Default constructor
	inline grTranslationd();
		/// see grNode
	const char * Name() const;
	void GL() const;

private:
	static const char * name;
};

	/// Scene rotation: the coordinate represents the axis of rotation
class grRotationf : public grTransform, public Coord3f
{
public:
		/// Default constructor
	inline grRotationf();
		/// see grNode
	const char * Name() const;
	void GL() const;

		/// Rotation angle
	GLfloat angle;

private:
	static const char * name;
};

	/// Scene rotation: the coordinate represents the axis of rotation
class grRotationd : public grTransform, public Coord3d
{
public:
		/// Default constructor
	inline grRotationd();
		/// see grNode
	const char * Name() const;
	void GL() const;

		/// Rotation angle
	GLdouble angle;

private:
	static const char * name;
};

	/// Scene scale: the coordinate represents the scale factors
class grScalef : public grTransform, public Coord3f
{
public:
		/// Default constructor
	inline grScalef();
		/// see grNode
	const char * Name() const;
	void GL() const;

private:
	static const char * name;
};

	/// Scene scale: the coordinate represents the scale factors
class grScaled : public grTransform, public Coord3d
{
public:
		/// Default constructor
	inline grScaled( void );
		/// see grNode
	const char * Name( void ) const;
	void GL( void ) const;

private:
	static const char * name;
};

	/// Generic Matrix transformation
class grTransformf : public grTransform, public grMatrixf
{
public:
	enum { MULTIPLY, LOAD };
		/// Default constructor
	inline grTransformf();
		/// see grNode
	const char * Name() const;
	void GL() const;

		/// Matrix transformation mode: MULTIPLY, LOAD
	int mode;

private:
	static const char * name;
};

	/// Generic Matrix transformation
class grTransformd : public grTransform, public grMatrixd
{
public:
	enum { MULTIPLY, LOAD };
		/// Default constructor
	inline grTransformd();
		/// see grNode
	const char * Name() const;
	void GL() const;

		/// Matrix transformation mode: MULTIPLY, LOAD
	int mode;

private:
	static const char * name;
};


inline grTransform::grTransform()
{
	mode = GL_MODELVIEW;
}


inline grTranslationf::grTranslationf()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}


inline grTranslationd::grTranslationd()
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
}


inline grRotationf::grRotationf()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
	angle = 0.0f;
}


inline grRotationd::grRotationd( )
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
	angle = 0.0;
}


inline grScalef::grScalef()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}


inline grScaled::grScaled()
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
}


inline grTransformf::grTransformf()
{
	mode = MULTIPLY;
}


inline grTransformd::grTransformd()
{
	mode = MULTIPLY;
}


#endif
