
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_VERTEX
#define GR_SHQN_VERTEX


#include <vector>
#include "base.h"
#include "node.h"
#include "status.h"
#include "coord.h"


using namespace std;

	/** Abstract base class for all coordinate nodes set.
	    This is the abstract base class from wich all coordinate node
		classes are derived. The coordinate node is a 'set' of
		coordinate (not a single coordinate).
		@see grPrimitive, STL
	 */
class grVertex : public grNode
{
public:
		/// return the size of the coordinate set
	virtual int Size(  ) const = 0;

	virtual void GL( const int i ) const = 0;

	void Render( grStatus & status );
	void Select( grStatus & status );
};

	/** @name Vertex sub-classes
	  */
	//@{

	/// bidimensional double precision coordinate set
class grVertex2d : public grVertex , public vector<Coord2d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		// see grVertex
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// bidimensional single precision coordinate set
class grVertex2f : public grVertex , public vector<Coord2f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size() const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// bidimensional integral coordinate set
class grVertex2i : public grVertex , public vector<Coord2i> 
{
public:
		/// see grNode
	const char * Name( ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};


	/// bidimensional single precision integral coordinate set
class grVertex2s : public grVertex , public vector<Coord2s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// tridimensional double precision coordinate set
class grVertex3d : public grVertex , public vector<Coord3d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};


	/// tridimensional single precision coordinate set
class grVertex3f : public grVertex , public vector<Coord3f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// tridimensional integral coordinate set
class grVertex3i : public grVertex , public vector<Coord3i> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// tridimensional single precision integral coordinate set
class grVertex3s : public grVertex , public vector<Coord3s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};


	/// quadridimensional double precision coordinate set
class grVertex4d : public grVertex , public vector<Coord4d> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// quadridimensional single precision coordinate set
class grVertex4f : public grVertex , public vector<Coord4f> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// quadridimensional integral coordinate set
class grVertex4i : public grVertex , public vector<Coord4i> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	/// quadridimensional single precision integral coordinate set
class grVertex4s : public grVertex , public vector<Coord4s> 
{
public:
		/// see grNode
	const char * Name(  ) const;
		/// see grVertex
	int Size(  ) const;
	void GL( const int i ) const;

private:
	static const char * name;
};

	//@}

#endif
